@@ -175,7 +175,11 @@ xml_parse_sux (const char *buf, int len)
 	mime_type = g_content_type_guess(NULL, (guchar *)buf, len, NULL);
 	//feeding parsed anything other than xml results in blocking delays
 	//it's possible we can relax parser by using xmlErrorFunc
-	if (!g_ascii_strncasecmp (mime_type, "application/", 12)) {
+	//UPDATE: add text/* - but exclude text/html I've seen huge delays because of this
+	//I doubt there'll be any text/html feeds
+	if (!g_ascii_strncasecmp (mime_type, "application/", 12)
+	   || (!g_ascii_strncasecmp (mime_type, "text/", 5)
+	   && g_ascii_strncasecmp (mime_type, "text/html", 9))) {
 		if (!sax) {
 			xmlInitParser();
 			sax = xmlMalloc (sizeof (xmlSAXHandler));
@@ -634,13 +638,16 @@ layer_find_innerhtml (xmlNodePtr node,
 		const char *match, const char *submatch,
 		gchar *fail)
 {
+	gchar *tmp;
+	xmlNodePtr tmpnode;
 	while (node!=NULL) {
-#ifdef RDF_DEBUG
-		xmlDebugDumpNode (stdout, node, 32);
-		printf("%s.\n", node->name);
-#endif
 		if (strcasecmp ((char *)node->name, match)==0 && node->children) {
-			return (gchar *)layer_find(node->children->next, submatch, fail);
+			tmpnode = node->children;
+			while (tmpnode) {
+				if ((tmp = (gchar *)layer_find(tmpnode, submatch, NULL)))
+					return tmp;
+				tmpnode = tmpnode->next;
+			}
 		}
 		node = node->next;
 	}
@@ -685,13 +692,19 @@ layer_query_find_prop (xmlNodePtr node,
 		const char *attrprop,
 		xmlChar *prop)
 {
-	node = node->children;
 	while (node!=NULL) {
-		if (!g_ascii_strcasecmp((gchar *)node->name, match)
-		&& (!g_ascii_strcasecmp((gchar *)xmlGetProp(node, attr), attrprop)
-		   || !xmlGetProp(node, attr)))
-					return (gchar *)xmlGetProp(node, prop);
-		node = node->next;
+		if (!g_ascii_strcasecmp((gchar *)node->name, match)) {
+			gchar *tprop = (gchar *)xmlGetProp(node, attr);
+			if (tprop) {
+				if (g_ascii_strcasecmp(tprop, attrprop)) {
+					xmlFree(tprop);
+					goto next;
+				}
+			}
+			xmlFree(tprop);
+		return (gchar *)xmlGetProp(node, prop);
+		}
+next:		node = node->next;
 	}
 	return NULL;
 }
@@ -824,7 +837,7 @@ tree_walk (xmlNodePtr root, RDF *r)
 				}
 				r->base = (gchar *)xmlGetProp(walk, (xmlChar *)"base");
 				if (!r->base)
-					r->base = layer_query_find_prop (walk,
+					r->base = layer_query_find_prop (walk->children,
 							"link",
 							(xmlChar *)"rel",
 							"alternate", (xmlChar *)"href");
@@ -983,10 +996,14 @@ parse_channel_line(xmlNode *top, gchar *feed_name, RDF *r, gchar **article_uid)
 	//we have to free this somehow
 	//<link></link>
 	link = g_strdup(layer_find (top, "link", NULL));		//RSS,
-	if (!link)								// <link href=>
-		link = (gchar *)layer_find_innerelement(
-				top, "link", "href",
-				g_strdup(_("No Information")));	//ATOM
+	if (!link) {								// <link href=>
+		if (!(link = layer_query_find_prop (top,
+				"link",
+				(xmlChar *)"rel",
+				"alternate", (xmlChar *)"href")))	//ATOM
+			link = g_strdup(_("No Information"));
+	}
+
 	id = (gchar *)layer_find (top, (gchar *)"id",				//ATOM
 			layer_find (top, (gchar *)"guid", NULL));		//RSS 2.0
 	feed = g_strdup_printf("%s\n", id ? id : link);
